%Same as the program corriidych.m, except that the transition matrix on returns is
%changed to non-benchmark values to check for robustness.

%Computes Gini, Tail Index, Quintiles and Lorentz curve, capital-labor ratio, plus mean, coeff of variation, and stdev of
%return on the stationary distribution plus other variables for internal check of consistency (mean and range of alpha)
%for various values of persistence and tax parameters for benchmark
%parameters of the model. 
%if the program runs through all taxes, zeta and b and epsilons it takes a very long
%time, more than a day, for each iteration, averaged overed over nn=80 iterations. So for shorter runs decrease nn, and shorten the loops for epsilon, 
% and taxes b and zeta. They are shortened below.

%correlated interest rate
%iid labor income

clc
clear all
tic;


sigma=2; %preference parameter
%gprime=0;
g=0.01; %growth of labor earnings during life
rhou=0.04; %discount rate
%b=0.0;
T=45; %length of life
chai=0.25;%bequest preference parameter
%zeta is the capital tax
%b is the estate tax


brmat=[0.08 0.12 0.15 0.32];%rate of return space
brmat=[0.065 0.12 0.15 0.27];



rstateN=length(brmat);

lt=0.0; %lt=labor tax
%ymat=(1-lt)*[0.3 1 2 5 10 30];
ymat=(1-lt)*[0.75 2.51 5.01 12.54 25.07 75.22];%labor earnings
ystateN=length(ymat);
yprob=[14/64 36/64 11/64 1/64 1/64 1/64];%labor earnings probabilities


ypaccum=zeros(1,ystateN+1);
ypaccum(1,1)=0;
for i=2:ystateN+1
    ypaccum(1,i)=ypaccum(1,i-1)+yprob(1,i-1);
end


bequsimuNumber=400000;
familyN=10000;

agegrid=[0:0.1:T];
%agegrid=[0:T];
ageNumber=length(agegrid);

N=familyN*ageNumber;


fid=fopen('simuresultdispcorr.txt','w');
fprintf(fid,'%s\n','--------------------------------------------------------------------------------');
fclose(fid);


LoopNumber=1;
taxesepsilonloop=1;

nn=1;%iterations


epsilon1 =0;% transition matrix parameter for intermediate values of r
epsilon2=.01; % transition matrix parameter for low r
%epsilon is transition matrix parameter for high r
  

for epsilon=0.02:0.01:0.02
   %transition matrix 
    
%rprob=[80/100+3*epsilon2 12/100-epsilon2 7/100-epsilon2 1/100-epsilon2;
 %  80/100-epsilon1 12/100+3*epsilon1 7/100-epsilon1 1/100-epsilon1;
  % 80/100-epsilon1 12/100-epsilon1 7/100+3*epsilon1 1/100-epsilon1;
  % 80/100-epsilon 12/100-epsilon 7/100-epsilon 1/100+3*epsilon];

rprob=[93/100+3*epsilon2 1/100-epsilon2 1/100-epsilon2 5/100-epsilon2;
 93/100-epsilon1 1/100+3*epsilon1 1/100-epsilon1 5/100-epsilon1;
 93/100-epsilon1 1/100-epsilon1 1/100+3*epsilon1 5/100-epsilon1;
  93/100-epsilon 1/100-epsilon 1/100-epsilon 5/100+3*epsilon];
  

rpaccum=zeros(rstateN,rstateN);
rpaccum(:,1)=rprob(:,1);
for i=2:rstateN
    rpaccum(:,i)=rpaccum(:,i-1)+rprob(:,i);
end


for b=0.2:0.05:0.2
    for zeta=0.15:0.05:0.15

rmat=(1-zeta)*brmat;

dq=0;
dgcl=0;

ddq=zeros(1,9);
ddgcl=zeros(1,2);
ddcy=0;
ddly=0;



for j=1:nn
   

%generate the sequence of r_n

rand('state',100*LoopNumber+1);
rshock=rand(1,bequsimuNumber);

rsimu=zeros(1,bequsimuNumber);
rsimu(1,1)=rmat(1,1);
for i=2:bequsimuNumber
    
      accumdf=rpaccum(find(rsimu(1,i-1)==rmat),:);
      rsimu(1,i)=rmat(1,find(rshock(1,i)<=accumdf,1));
    
end

%generate the sequence of y_n

rand('state',100*LoopNumber+2);
yshock=rand(1,bequsimuNumber);

ysimu=zeros(1,bequsimuNumber);
for i=1:ystateN
    ysimu=ysimu+ymat(1,i)*(ypaccum(1,i)<yshock & yshock<=ypaccum(1,i+1));
end

%gnenerate alpha_n and beta_n

A=rsimu-(rsimu-rhou)/sigma;
B=chai^(1/sigma)*(1-b)^((1-sigma)/sigma);

alpha=(1-b)*exp(rsimu*T).*A*B./(exp(A*T)+A*B-1);
beta=alpha.*ysimu.*((exp((g-rsimu)*T)-1)./(g-rsimu));


%generate z_n

z=zeros(1,bequsimuNumber);
for i=1:bequsimuNumber-1
    z(1,i+1)=alpha(1,i)*z(1,i)+beta(1,i);
end

%generate the initial wealth

familyN=10000;

initialwealth=z(1,bequsimuNumber-familyN+1:bequsimuNumber);


%gnenerate the random interest rate and labor income of alive families

ralivesimu=rsimu(1,bequsimuNumber-familyN+1:bequsimuNumber)';


rand('state',100*LoopNumber+3);
yaliveshock=rand(familyN,1);

yalivesimu=zeros(familyN,1);
for i=1:ystateN
    yalivesimu=yalivesimu+ymat(1,i)*(ypaccum(1,i)<yaliveshock & yaliveshock<=ypaccum(1,i+1));
end

%generate sigmaw and sigmay

%agegrid=[0:0.1:T];
%agegrid=[0:T];
%ageNumber=length(agegrid);

Aalive=ralivesimu-(ralivesimu-rhou)/sigma;
Balive=chai^(1/sigma)*(1-b)^((1-sigma)/sigma);

sigmaw=exp(ralivesimu*agegrid) ...
    .*(exp(Aalive*(T-agegrid))+(Aalive*Balive-1)*ones(1,ageNumber))./((exp(Aalive*T)+Aalive*Balive-1)*ones(1,ageNumber));

sigmay=(exp((g-ralivesimu)*T)-1)./(g-ralivesimu)*ones(1,ageNumber) ...
    .*(sigmaw-exp(ralivesimu*agegrid).*(exp((ralivesimu-g)*(T-agegrid))-1)./((exp((ralivesimu-g)*T)-1)*ones(1,ageNumber)));

%simulate wealth

wealthsimu=diag(initialwealth)*sigmaw+diag(yalivesimu)*sigmay;



%compute Lorenz curve

%N=familyN*ageNumber;

wdata=reshape(wealthsimu,1,N);

wealth=sort(wdata);

accumw=zeros(1,N);
accumw(1,1)=wealth(1,1);
for i=1:N-1
    accumw(1,i+1)=accumw(1,i)+wealth(1,i+1);
end

lorenz=accumw/sum(wealth);
%cv=std(wealth)/mean(wealth);
%wealth1=wealth;


%Gini coefficient
%figure(1)
%plot([1:N]/N,lorenz);
%title('Lorenz Curve');
Gini=1-2*sum(lorenz)*1/N;
%disp('Gini coefficient=');
%disp(Gini);



%compute the ratio of capital income and labor income

capitalincome=sum(sum(diag(ralivesimu)*wealthsimu))/N;

%laborincome=ymat*yprob'*1/T*(exp(g*T)-1)/g;

laborincome=sum(sum(yalivesimu*exp(g*agegrid)))/N;


%disp('capital income/labor income=');
%disp(capitalincome/laborincome);


dq=[1-lorenz(1,floor(0.99*N)),lorenz(1,floor(0.99*N))-lorenz(1,floor(0.95*N)), ...
    lorenz(1,floor(0.95*N))-lorenz(1,floor(0.90*N)), 1-lorenz(1,floor(0.80*N)), ...
    lorenz(1,floor(0.80*N))-lorenz(1,floor(0.60*N)), lorenz(1,floor(0.60*N))-lorenz(1,floor(0.40*N)), ...
    lorenz(1,floor(0.40*N))-lorenz(1,floor(0.20*N)), lorenz(1,floor(0.2*N)), Gini];

dgcl=[Gini, capitalincome/laborincome];

ddq=[ddq; dq];
ddgcl=[ddgcl; dgcl];
ddcy=[ddcy capitalincome];
ddly=[ddly laborincome];


LoopNumber=LoopNumber+1;

end %end of j


%compute the tail index

A1=rmat-(rmat-rhou)./sigma;
B1=chai^(1/sigma)*(1-b)^((1-sigma)/sigma);

alpha1=(1-b).*exp(rmat*T).*A1.*B1./(exp(A1*T)+A1.*B1-1);

options = optimset('TolFun',1e-12,'TolX',1e-12);
[muh,fval,exitflag]=fzero(@(mu) max(eig(diag(alpha1)^mu*rprob'))-1,2,options);

dddq=sum(ddq)/nn;
dddgcl=sum(ddgcl)/nn;
dddcy=sum(ddcy)/nn;
dddly=sum(ddly)/nn;

bz=[b zeta epsilon nn dddcy dddly];

meanalpha=rprob^1000*alpha1';

S=rprob^1000;

meanr=brmat*S(1,:)';
sdr=(((brmat-meanr).^2)*S(1,:)')^.5;
coeffvar=sdr/meanr;


fprintf('%s\n','           b          zeta       epsilon         nn        capy        labory');
fprintf('%12.3f %12.3f %12.3f %12d %12.3f %12.3f\n',bz);

fprintf('%s\n','                             alpha ');
fprintf('%12.3f',[alpha1]);
fprintf('\n');

fprintf('%s\n','       meanalpha     mu');
fprintf('%12.3f',[meanalpha(1,1), muh]);
fprintf('\n');

fprintf('%s\n','       Gini      Capy/Laby');
fprintf('%12.3f',dddgcl);
fprintf('\n');

fprintf('%s\n','       meanr     stdevr     coeff of var');
fprintf('%12.3f', [meanr sdr coeffvar] );
fprintf('\n');


fprintf('%s\n','      99-100      95-99      90-95      80-100     60-80     40-60       20-40     0-20        Gini');
fprintf('%11.3f',dddq);
fprintf('\n');
fprintf('%s\n',' For US- from Rodriguez, Diaz-Gimenez, Quadrini, Rios-Rull, Fed Minn Quarterly, Summer 2002, table 7 ');
fprintf('%s\n','    99-100     95-99       90-95       80-100    60-80     40-60      20-40     0-20        Gini ');
fprintf('%s\n','    0.347      0.231       0.113       0.817     0.122      0.05      0.013    -0.003      0.803');
fprintf('%s\n','--------------------------------------------------------------------------------');


fid=fopen('simuresultdispcorr.txt','a');
fprintf(fid,'%s\n','           b          zeta       epsilon         nn        capy        labory');
fprintf(fid,'%12.3f %12.3f %12.3f %12d %12.3f %12.3f\n',bz);
fprintf(fid,'%s\n','                    alpha                                            meanalpha     mu');
fprintf(fid,'%12.3f',[alpha1, meanalpha(1,1), muh]);
fprintf(fid,'\n');
fprintf(fid,'%s\n','        Gini      Capy/Laby');
fprintf(fid,'%12.3f',dddgcl);
fprintf(fid,'\n');
fprintf(fid,'%s\n','      99-100      95-99      90-95      80-100     60-80     40-60       20-40     0-20        Gini');
fprintf(fid,'%11.3f',dddq);
fprintf(fid,'\n');
fprintf(fid,'%s\n',' For US- from Rodriguez, Diaz-Gimenez, Quadrini, Rios-Rull, Fed Minn Quarterly, Summer 2002, table 7 ');
fprintf(fid,'%s\n','    99-100     95-99       90-95       80-100    60-80     40-60      20-40     0-20        Gini ');
fprintf(fid,'%s\n','    0.347      0.231       0.113       0.817     0.122      0.05      0.013    -0.003      0.803');
fprintf(fid,'%s\n','--------------------------------------------------------------------------------');
fclose(fid);


bzresult(taxesepsilonloop,:)=bz;
dddgclresult(taxesepsilonloop,:)=dddgcl;
dddqresult(taxesepsilonloop,:)=dddq;

taxesepsilonloop=taxesepsilonloop+1;
    
    
    end; %end of zeta
end; %end of b

end; %end of epsilon

%mugrid=[0.01:0.01:2];
%Nmu=length(mugrid);
%fmu=zeros(1,Nmu);
%for i=1:Nmu
%    fmu(1,i)=max(eig(diag(alpha1)^(mugrid(1,i))*rprob'))-1;
%end
%plot(mugrid,fmu);


t=toc;
hh=floor(t/3600);
mn=floor((t-hh*3600)/60);
fprintf('Time elapse is %d hour(s) %d minute(s).\n',hh,mn);

fid=fopen('simuresultdispcorr.txt','a');
fprintf(fid,'Time elapse is %d hour(s) %d minute(s).\n',hh,mn);
fclose(fid);

save simuresultcorr;

